@@ -91,6 +91,7 @@ class open:
             self.file = _open_(name, mode, encoding=encoding, errors=errors)
         self.encoding = encoding
         self.errors = errors
+        self.iterator = None
 
     def __iter__(self):
         for l in self.file:
@@ -98,6 +99,14 @@ class open:
                 l = convert_to_unicode(l, self.encoding, self.errors)
             yield l
 
+    def next(self):
+        if self.iterator is None:
+            self.iterator = self.__iter__()
+        return next(self.iterator)
+
+    def __next__(self):
+        return self.next()
+
     def read(self):
         text = self.file.read()
         if self.encoding:
@@ -364,11 +373,17 @@ class AutoRegressiveDecoder(object):
     def wraps(default_rtype='probas', use_states=False):
         """用来进一步完善predict函数
         目前包含：1. 设置rtype参数，并做相应处理；
-                  2. 确定states的使用，并做相应处理。
+                  2. 确定states的使用，并做相应处理；
+                  3. 设置温度参数，并做相应处理。
         """
         def actual_decorator(predict):
             def new_predict(
-                self, inputs, output_ids, states, rtype=default_rtype
+                self,
+                inputs,
+                output_ids,
+                states,
+                temperature=1,
+                rtype=default_rtype
             ):
                 assert rtype in ['probas', 'logits']
                 prediction = predict(self, inputs, output_ids, states)
@@ -377,7 +392,13 @@ class AutoRegressiveDecoder(object):
                     prediction = (prediction, None)
 
                 if default_rtype == 'logits':
-                    prediction = (softmax(prediction[0]), prediction[1])
+                    prediction = (
+                        softmax(prediction[0] / temperature), prediction[1]
+                    )
+                elif temperature != 1:
+                    probas = np.power(prediction[0], 1.0 / temperature)
+                    probas = probas / probas.sum(axis=-1, keepdims=True)
+                    prediction = (probas, prediction[1])
 
                 if rtype == 'probas':
                     return prediction
@@ -388,16 +409,16 @@ class AutoRegressiveDecoder(object):
 
         return actual_decorator
 
-    def predict(self, inputs, output_ids, states=None, rtype='logits'):
+    def predict(self, inputs, output_ids, states=None):
         """用户需自定义递归预测函数
-        说明：rtype为字符串logits或probas，用户定义的时候，应当根据rtype来
-              返回不同的结果，rtype=probas时返回归一化的概率，rtype=logits时
-              则返回softmax前的结果或者概率对数。
+        说明：定义的时候，需要用wraps方法进行装饰，传入default_rtype和use_states，
+             其中default_rtype为字符串logits或probas，probas时返回归一化的概率，
+             rtype=logits时则返回softmax前的结果或者概率对数。
         返回：二元组 (得分或概率, states)
         """
         raise NotImplementedError
 
-    def beam_search(self, inputs, topk, states=None, min_ends=1):
+    def beam_search(self, inputs, topk, states=None, temperature=1, min_ends=1):
         """beam search解码
         说明：这里的topk即beam size；
         返回：最优解码序列。
@@ -406,7 +427,7 @@ class AutoRegressiveDecoder(object):
         output_ids, output_scores = self.first_output_ids, np.zeros(1)
         for step in range(self.maxlen):
             scores, states = self.predict(
-                inputs, output_ids, states, 'logits'
+                inputs, output_ids, states, temperature, 'logits'
             )  # 计算当前得分
             if step == 0:  # 第1步预测后将输入重复topk次
                 inputs = [np.repeat(i, topk, axis=0) for i in inputs]
@@ -436,7 +457,14 @@ class AutoRegressiveDecoder(object):
         return output_ids[output_scores.argmax()]
 
     def random_sample(
-        self, inputs, n, topk=None, topp=None, states=None, min_ends=1
+        self,
+        inputs,
+        n,
+        topk=None,
+        topp=None,
+        states=None,
+        temperature=1,
+        min_ends=1
     ):
         """随机采样n个结果
         说明：非None的topk表示每一步只从概率最高的topk个中采样；而非None的topp
@@ -448,7 +476,7 @@ class AutoRegressiveDecoder(object):
         results = []
         for step in range(self.maxlen):
             probas, states = self.predict(
-                inputs, output_ids, states, 'probas'
+                inputs, output_ids, states, temperature, 'probas'
             )  # 计算当前概率
             probas /= probas.sum(axis=1, keepdims=True)  # 确保归一化
             if step == 0:  # 第1步预测后将结果重复n次
@@ -624,18 +652,16 @@ class WebServing(object):
             kwargs = {}
             for key, value in arguments.items():
                 if method == 'GET':
-                    result = self.bottle.request.GET.get(key)
+                    result = self.bottle.request.GET.getunicode(key)
                 else:
-                    result = self.bottle.request.POST.get(key)
+                    result = self.bottle.request.POST.getunicode(key)
                 if result is None:
                     if value[1]:
                         outputs['code'] = 1
                         outputs['desc'] = 'lack of "%s" argument' % key
                         return json.dumps(outputs, ensure_ascii=False)
                 else:
-                    if value[0] is None:
-                        result = convert_to_unicode(result)
-                    else:
+                    if value[0] is not None:
                         result = value[0](result)
                     kwargs[key] = result
             try:
